import React, { useEffect, useRef, useState } from 'react';
import * as d3 from 'd3';

export type SingleStackBarplotProps = {
  data: { name: string; value: number; to?: string; from?: string }[];
  title: string;
};

interface StackDataItem {
  name: string;
  value: number;
  startValue: number;
  endValue: number;
  width: number;
}

const stack = function (data: { name: string; value: number }[], maxWidth: number): StackDataItem[] {
  const total = d3.sum(data, (d) => d.value);
  let accumulatedWidth = 0;
  return data.map((d) => {
    const width = maxWidth * (d.value / total);
    const startValue = accumulatedWidth;
    accumulatedWidth += width;
    const endValue = accumulatedWidth;
    return { name: d.name, value: d.value, startValue, endValue, width };
  });
};
export const SingleStackBarplot = ({ data, title }: SingleStackBarplotProps) => {
  const svgRef = useRef<SVGSVGElement | null>(null);
  const [dimensions, setDimensions] = useState({ width: 0, height: 0 });

  useEffect(() => {
    if (!svgRef.current) return;

    const widthSVG: number = +svgRef.current.clientWidth;
    const heightSVG: number = 0.13 * widthSVG;

    setDimensions({ width: widthSVG, height: heightSVG });

    const marginPercentage = {
      top: 0.1,
      right: 0.2,
      bottom: 0.4,
      left: 0.2,
    };
    const margin = {
      top: marginPercentage.top * heightSVG,
      right: marginPercentage.right * heightSVG,
      bottom: marginPercentage.bottom * heightSVG,
      left: marginPercentage.left * heightSVG,
    };

    const svgPlot = d3.select(svgRef.current);

    svgPlot.selectAll('*').remove();
    const groupMargin = svgPlot.append('g').attr('transform', `translate(${margin.left},${margin.top})`);
    const widthSVGwithinMargin: number = widthSVG - margin.left - margin.right;
    const heightSVGwithinMargin: number = heightSVG - margin.top - margin.bottom;

    const stackData = stack(data, widthSVGwithinMargin);

    const configPlot = {
      strokeWidth: 1,
      strokeColor: 'secondary-300',
    };

    groupMargin
      .selectAll<SVGRectElement, d3.Bin<string, number>>('singleStackBarplot')
      .data(stackData)
      .enter()
      .append('rect')
      .style('fill', 'white')
      .style('stroke', 'none')
      .style('stroke-width', '1')
      .attr('class', (d, i) => {
        if (title === 'Edges') {
          return `wdb_${d.name} wdbEdge_${data[i].to} wdbEdge_${data[i].from}`;
        } else {
          return `wdb_${d.name}`;
        }
      })
      .attr('x', (d) => d.startValue)
      .attr('y', 0.0)
      .attr('width', (d) => d.width)
      .attr('height', heightSVGwithinMargin)
      .on('mouseover', function (d) {
        // cursor
        d3.select(this).style('cursor', 'pointer');

        // rects
        const classNames = d3.select(this).attr('class').split(' ');
        const className = classNames[0];
        d3.select('.' + className).style('fill', 'hsl(var(--clr-acc))');

        if (title == 'Nodes') {
          const classNameFromNode2Edge = className.split('_');
          // rect edges
          d3.selectAll('.wdbEdge_' + classNameFromNode2Edge[1]).style('fill', 'hsl(var(--clr-acc))');

          // list nodes
          const liElements = document.querySelectorAll(`.divPills.wdb_${classNameFromNode2Edge[1]}`);
          //console.log('liElements ', liElements);

          liElements.forEach((liElement) => {
            (liElement as HTMLElement).style.borderColor = 'hsl(var(--clr-acc))';
          });
        } else {
          const classNameFromEdege2Node1 = classNames[1].split('_');
          const classNameFromEdege2Node2 = classNames[2].split('_');

          // rect nodes
          d3.select('.wdb_' + classNameFromEdege2Node1[1]).style('fill', 'hsl(var(--clr-acc))');
          d3.select('.wdb_' + classNameFromEdege2Node2[1]).style('fill', 'hsl(var(--clr-acc))');

          //const selector1 = `[class*=wdb_${classNameFromEdege2Node1[1]}]`;
          //const liElements1 = document.querySelectorAll(selector1);
          const liElements1 = document.querySelectorAll(`.divPills.wdb_${classNameFromEdege2Node1[1]}`);
          const liElement1 = liElements1[0];
          (liElement1 as HTMLElement).style.borderColor = 'hsl(var(--clr-acc))';

          //const selector2 = `[class*=wdb_${classNameFromEdege2Node2[1]}]`;
          //const liElements2 = document.querySelectorAll(selector2);
          const liElements2 = document.querySelectorAll(`.divPills.wdb_${classNameFromEdege2Node2[1]}`);

          const liElement2 = liElements2[0];
          (liElement2 as HTMLElement).style.borderColor = 'hsl(var(--clr-acc))';
        }

        // li

        const selector = `[class*="${className}"]`;
        const liElements = document.querySelectorAll(selector);
        liElements.forEach((liElement) => {
          (liElement as HTMLElement).style.borderColor = 'hsl(var(--clr-acc))';
        });
      })
      .on('mouseout', function (d) {
        // cursor
        d3.select(this).style('cursor', 'default');

        // rects
        const classNames = d3.select(this).attr('class').split(' ');
        const className = classNames[0];
        d3.selectAll('.' + className).style('fill', 'white');

        d3.selectAll('.divPills').style('border-color', 'white');
        d3.select('.nodesInformation').selectAll('rect:not(.wrapingRect)').style('fill', 'white');
        d3.select('.edgesInformation').selectAll('rect:not(.wrapingRect)').style('fill', 'white');
      });

    groupMargin
      .selectAll<SVGLineElement, StackDataItem>('endValueLine')
      .data(stackData.slice(0, -1))
      .enter()
      .append('line')
      .attr('class', 'fill-none stroke-secondary-700')
      .style('stroke-width', configPlot.strokeWidth)
      .attr('x1', (d) => d.endValue)
      .attr('y1', 0)
      .attr('x2', (d) => d.endValue)
      .attr('y2', heightSVGwithinMargin);

    groupMargin
      .append('rect')
      .attr('x', 0.0)
      .attr('y', 0.0)
      .style('stroke-width', configPlot.strokeWidth)
      .attr('width', widthSVGwithinMargin)
      .attr('height', heightSVGwithinMargin)
      .attr('class', 'wrapingRect fill-none stroke-secondary-700');

    groupMargin
      .append('text')
      .attr('x', 0)
      .style('stroke', 'none')
      .style('fill', 'hsl(var(--clr-sec--500))')
      .attr('class', 'font-mono')
      .attr('y', heightSVGwithinMargin + margin.bottom / 2)
      .attr('text-anchor', 'middle')
      .attr('alignment-baseline', 'middle')
      .text('0');

    groupMargin
      .append('text')
      .attr('x', widthSVGwithinMargin)
      .attr('y', heightSVGwithinMargin + margin.bottom / 2)
      .style('stroke', 'none')
      .style('fill', 'hsl(var(--clr-sec--500))')
      .attr('class', 'font-mono')
      .attr('text-anchor', 'end')
      .attr('alignment-baseline', 'middle')
      .text(() => {
        const cumulativeValue = stackData.reduce((accumulator, currentValue) => accumulator + currentValue.value, 0);
        return `Number of ${title}: ${cumulativeValue}`;
      });

    groupMargin.selectAll('.wdb_text_axis').attr('class', 'font-mono font-small text-secondary-500').style('stroke', 'none');
    /*
    svgPlot
      .append('rect')
      .attr('x', 0.0)
      .attr('y', 0.0)
      .attr('width', widthSVG)
      .attr('height', heightSVG)
      .attr('rx', 0)
      .attr('ry', 0)
      .attr('class', 'fill-none stroke-secondary-300');
      */
  }, [data]);

  return (
    <div className="svg">
      <svg
        ref={svgRef}
        className="container"
        width="100%"
        height={dimensions.height}
        preserveAspectRatio="xMidYMid meet"
        viewBox={`0 0 ${dimensions.width} ${dimensions.height}`}
      ></svg>
    </div>
  );
};

export default SingleStackBarplot;
