import React, { useMemo, useState, useEffect } from 'react';
import { EntityPill, RelationPill } from '@graphpolaris/shared/lib/components/pills/Pill';
import { select, selectAll } from 'd3';

export type Item = {
  name: string;
  value: number;
  to?: string;
  from?: string;
};

export type ColumnedListProps = {
  items: Item[];
  maxHeight: string;
  typePill: string;
};

const ColumnedList: React.FC<ColumnedListProps> = ({ items, maxHeight, typePill }) => {
  const maxElementsPerColumn = 5;
  const maxNumberOfColumns = 3;

  const [elementsPerColumn, setElementsPerColumn] = useState(maxElementsPerColumn);

  useEffect(() => {
    const totalMaxElements = maxElementsPerColumn * maxNumberOfColumns;
    if (items.length > totalMaxElements) {
      const newElementsPerColumn = Math.ceil(items.length / maxNumberOfColumns);
      setElementsPerColumn(newElementsPerColumn);
    } else {
      setElementsPerColumn(maxElementsPerColumn);
    }
  }, [items.length]);

  const numColumns = useMemo(
    () => Math.min(Math.ceil(items.length / elementsPerColumn), maxNumberOfColumns),
    [items.length, elementsPerColumn],
  );

  const columns: Item[][] = useMemo(() => {
    const columnsArray: Item[][] = [];
    for (let i = 0; i < numColumns; i++) {
      columnsArray.push(items.slice(i * elementsPerColumn, (i + 1) * elementsPerColumn));
    }
    return columnsArray;
  }, [items, numColumns, elementsPerColumn]);
  return (
    <div
      className={`border-gray-300 border bg-white flex h-52 flex-col lg:flex-row lg:flex-wrap overflow-x-hidden ${numColumns > 3 ? 'overflow-y-auto' : ''}`}
    >
      {columns.map((column, index) => (
        <div key={index} className="flex-1 py-2 px-4 max-w-full h-full">
          <div className={`max-h-${maxHeight}`}>
            <ul className="flex flex-col h-full">
              {column.map((item, idx) => (
                <div key={idx} className={`flex justify-center m-1 div${typePill}`}>
                  <li
                    //className={`m-0 text-xw-full flex justify-center items-center ${typePill == 'Nodes' ? `wdb_${item.name}` : `wdb_${item.name} wdb_${item.to} wdb_${item.from}`}`}
                    className={`m-0 text-xw-full flex justify-center items-center `}
                    onMouseEnter={(event) => {
                      // rect elements own
                      const rectElements = document.querySelectorAll(`rect.wdb_${item.name}`);

                      rectElements.forEach((rectElement) => {
                        (rectElement as HTMLElement).style.fill = 'hsl(var(--clr-acc))';
                      });

                      // li elements both
                      const liElements = document.querySelectorAll(`.wdb_${item.name}`);
                      liElements.forEach((liElement) => {
                        (liElement as HTMLElement).style.borderColor = 'hsl(var(--clr-acc))';
                        (liElement as HTMLElement).style.cursor = 'pointer';
                      });

                      // rect other
                      if (typePill === 'Nodes') {
                        const rectElementsOther = document.querySelectorAll(`rect.wdbEdge_${item.name}`);

                        rectElementsOther.forEach((rectElement) => {
                          (rectElement as HTMLElement).style.fill = 'hsl(var(--clr-acc))';
                        });
                      } else {
                        const hoveredElement = event.target as HTMLElement;
                        const hoveredElementClass = hoveredElement.className;

                        const classNamesArray = hoveredElementClass.split(' ');
                        const wdbClassNames = classNamesArray.filter((className) => className.startsWith('wdb_'));

                        console.log('wdbClassNames ', wdbClassNames);

                        // rect nodes 1
                        const rectElements1 = document.querySelectorAll(`rect.${wdbClassNames[1]}`);
                        rectElements1.forEach((rectElement1) => {
                          (rectElement1 as HTMLElement).style.fill = 'hsl(var(--clr-acc))';
                        });

                        // rect nodes 2
                        const rectElements2 = document.querySelectorAll(`rect.${wdbClassNames[2]}`);
                        rectElements2.forEach((rectElement2) => {
                          (rectElement2 as HTMLElement).style.fill = 'hsl(var(--clr-acc))';
                        });

                        // li nodes
                        //console.log('wdbClassNames', wdbClassNames, wdbClassNames.length);
                        if (wdbClassNames.length != 0) {
                          const liElements1 = document.querySelectorAll(`.divPills.${wdbClassNames[1]}`);
                          (liElements1[0] as HTMLElement).style.borderColor = 'hsl(var(--clr-acc))';

                          const liElements2 = document.querySelectorAll(`.divPills.${wdbClassNames[2]}`);
                          (liElements2[0] as HTMLElement).style.borderColor = 'hsl(var(--clr-acc))';
                        }
                      }
                    }}
                    onMouseLeave={(event) => {
                      // rect elements own
                      const rectElements = document.querySelectorAll(`rect.wdb_${item.name}`);

                      rectElements.forEach((rectElement) => {
                        (rectElement as HTMLElement).style.fill = 'white';
                      });

                      // li elements both
                      const liElements = document.querySelectorAll(`.wdb_${item.name}`);
                      liElements.forEach((liElement) => {
                        (liElement as HTMLElement).style.borderColor = 'white';
                        (liElement as HTMLElement).style.cursor = 'pointer';
                      });

                      select('.nodesInformation').selectAll('rect:not(.wrapingRect)').style('fill', 'white');
                      select('.edgesInformation').selectAll('rect:not(.wrapingRect)').style('fill', 'white');

                      // rect other
                      if (typePill === 'Nodes') {
                        const rectElementsOther = document.querySelectorAll(`rect.wdbEdge_${item.name}`);

                        rectElementsOther.forEach((rectElement) => {
                          (rectElement as HTMLElement).style.fill = 'white';
                        });
                      } else {
                        const hoveredElement = event.target as HTMLElement;
                        const hoveredElementClass = hoveredElement.className;

                        const classNamesArray = hoveredElementClass.split(' ');
                        const wdbClassNames = classNamesArray.filter((className) => className.startsWith('wdb_'));

                        // li nodes
                        if (wdbClassNames.length != 0) {
                          const liElements1 = document.querySelectorAll(`.divPills.${wdbClassNames[1]}`);
                          liElements1.forEach((liElement1) => {
                            (liElement1 as HTMLElement).style.borderColor = 'white';
                          });

                          const liElements2 = document.querySelectorAll(`.divPills.${wdbClassNames[2]}`);
                          liElements2.forEach((liElement2) => {
                            (liElement2 as HTMLElement).style.borderColor = 'white';
                          });
                        }
                      }
                    }}
                  >
                    <div
                      className={`border-2 border-white rounded divPills ${typePill == 'Nodes' ? `wdb_${item.name}` : `wdb_${item.name} wdb_${item.to} wdb_${item.from}`}`}
                    >
                      {typePill == 'Nodes' ? (
                        <EntityPill title={item.name + ' : ' + item.value} />
                      ) : (
                        <RelationPill title={item.name + ' : ' + item.value} />
                      )}
                    </div>
                  </li>
                </div>
              ))}
            </ul>
          </div>
        </div>
      ))}
    </div>
  );
};

export default ColumnedList;
